// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package model

import "time"

type ExpenseRequest struct {
	Amount       Money         `json:"money"         binding:"required,number"`
	PayerIDs     []int         `json:"payer_ids"     binding:"required"`
	RecipientIDs []int         `json:"recipient_ids" binding:"required"`
	EventID      int           `json:"event_id"      binding:"required"`
	Detail       ExpenseDetail `json:"detail"`
}

type ExpenseEntity struct {
	ID int

	Amount   int
	Currency string
	EventID  int

	// ExpenseDetail
	Name  string
	Place string

	CreatedAt time.Time
	UpdatedAt time.Time
}

type ExpenseDetail struct {
	Name  string `json:"name"`
	Place string `json:"place"`
}

type Expense struct {
	ID int

	Amount Money

	// Lazy aggregate using Transaction join
	PayerIDs []int

	// Lazy aggregate using Transaction join
	RecipientIDs []int

	EventID int
	Detail  ExpenseDetail

	CreatedAt time.Time
	UpdatedAt time.Time
}
